c      'search BPKM.for'
c
c      This is the source code program for the model simulations used in the following papers: 
c
c      **************************************************************************************************************   
c      *                                                                                                            *
c      *   The follwoing paper should be referenced in publications which use this code.                           *
c      *                                                                                                            *  
c      **************************************************************************************************************
c
c      Pan, L., Matsui, Y., Matsushita, T. and Shirasaki, N., 
c      Superiority of wet-milled over dry-milled superfine powdered activated carbon for adsorptive 2-methylisoborneol removal,
c      Water Research, 2016.
c
c      The execution file may be obtained from the corresponding author of the papers. 
c
c      This gives time change of adsorbate concentration in batch adsorption of single solute system. 
c      film mass transport (kf), surface diffusion (Ds) , tortuosity (Tort) , and micro pore diffusion (ks)
c      are the main kinetic parameters.
c      These paramter values can be searched for best fit to experimental data.
c
c      Pore diffusion coeficient value is calculated from tortuosity (Tort) value by :
c        	 Dp = diffusivity * porosity / tortuosity, rdp=rdpd*ep/tort
c
c      The diffusivity is calculated by the corelation equation of the Hayduk-Laudie method
c          diffusivity =  1.326e-4 / viscosity**1.14 / molecular volume **0.589, rdpd=1.326e-4/vis**1.14/rmvol**0.589
c      
c      Adsorption isotherm is described by Freundlich equation 
c
c      Single adsorbate system
c
c      The execution requires 
c                an input file named "BPKM_input.txt" 
c          and a file which contains W and B matrixes.
c
c         Yoshihiko Matsui
c         Hokkaido University, 7.7.2016 
c
c	    Orthogonal Collocation method
c              This program requires IMSL subroutine " divpag "
c
c
	 IMPLICIT DOUBLE PRECISION (A-H,O-Z)

	 parameter (ndim4=15)
	 parameter (ndim6=13)
	 parameter (ntest=15)

c      dimension of search
	 parameter (ns=2)

c	 -------for divpag----------
	 dimension aaa(1,1)
c      -------------------------
       
       dimension acdose(ndim6),acdia(ntest,ndim6),radi(ndim6)
       dimension wr(ndim4),br(ndim4,ndim4)
       dimension acdosed(ntest,ndim6)
       dimension cinf(ntest),cc(ntest),nd(ntest),tt(ntest,10)
     &          ,rc(ntest,10),rfkd(ntest),rnd(ntest)
     &          ,psd(ntest),epd(ntest),d50t(ntest),d50a(ntest)
       dimension iparam(7)
       dimension rparam(7)
       dimension xx(ns), xguess(ns), xscale(ns), xs(ns)

	 common/b1/nac
	 common/b2/acdose,acdia,radi
       common/b3/rfk,rn,pfk,ps,ep,co,rkf,rds,rdp,tort,
     &           rdss,rks,ceff,rceff
	 common/b4/ncol
       common/b5/wr,br
	 common/b33/rdpd,ps1,ep1,beta
	 common/b6/cinf,cc,tt,rc,psd,epd,d50t,d50a
	 common/b7/ndata,nd
	 common/b8/acdosed,tol,rfkd,rnd
	 common/b9/param11
	 common/b88/miter
       common/b10/iflag,nss

       external fcn	                     
       external fcn2,fcnj,umach,sset

       data rfk,pfk,rn,co,rkf,rds,rdp,rdpd,rks,ceff,rceff/11*0.0/
       data iflag/0/

	open(10,file='search BPKM input.txt',status='unknown')
      open(20,file='search BPKM output.txt',status='unknown')

	 if(ndim4.eq.10) open(11,file='col10.txt',status='unknown')
	 if(ndim4.eq.11) open(11,file='col11.txt',status='unknown')
	 if(ndim4.eq.12) open(11,file='col12.txt',status='unknown')
	 if(ndim4.eq.13) open(11,file='col13.txt',status='unknown')
 	 if(ndim4.eq.14) open(11,file='col14.txt',status='unknown')
 	 if(ndim4.eq.15) open(11,file='col15.txt',status='unknown')
	 if(ndim4.eq.16) open(11,file='col16.txt',status='unknown')
	 if(ndim4.eq.17) open(11,file='col17.txt',status='unknown')
	 if(ndim4.eq.18) open(11,file='col18.txt',status='unknown')
	 if(ndim4.eq.19) open(11,file='col19.txt',status='unknown')

       read(10,*) nss
	 read(10,*) miter
	 read(10,*) tol
	 read(10,*) param11
	 read(10,*) nac
	 read(10,*) ps1
	 read(10,*) ep1
	 read(10,*) beta
	 read(10,*) vis
	 read(10,*) tort
	 read(10,*) pfk
	 read(10,*)	rmvol
       read(10,*) rkf
       read(10,*) rdss
	 read(10,*) rks

       read(10,*) ndata
	 do 302 i=1,ndata
	 read(10,*) cinf(i)
	 read(10,*) cc(i)
	 read(10,*) d50t(i)
	 read(10,*) d50a(i)
	 read(10,*) (acdosed(i,j), j=1,nac)
       read(10,*) (acdia(i,j), j=1,nac)
       read(10,*) rfkd(i)      
	 read(10,*) rnd(i)
	 read(10,*) nd(i)
	 do 303 j=1,nd(i)
	 read(10,*) tt(i,j),rc(i,j)

       psd(i)=ps1*(d50t(i)/d50a(i))**beta 
	 epd(i)=1.0-psd(i)/ps1*(1.0-ep1)

  303  continue
  302  continue
c
       write(20,*) miter
	 write(20,*) tol
	 write(20,*) param11
	 write(20,*) nac
	 write(20,*) ps1
	 write(20,*) ep1
	 write(20,*) beta
 	 write(20,*) vis
	 write(20,*) tort
	 write(20,*) pfk
	 write(20,*) rmvol
       write(20,*) rkf
       write(20,*) rdss
       write(20,*) rks
       write(20,*)
	 	 	 
	 rdpd=1.326e-4/vis**1.14/rmvol**0.589
	 
	 write(20,*) rdpd
	 write(20,*)

       write(20,*) ndata
	 do 312 i=1,ndata
	 write(20,*) cinf(i)
	 write(20,*) cc(i)
	 write(20,*) d50t(i)
	 write(20,*) d50a(i)
	 write(20,*) (acdosed(i,j), j=1,nac)
       write(20,*) (acdia(i,j), j=1,nac)
       write(20,*) rfkd(i)      
	 write(20,*) rnd(i)
	 write(20,*) nd(i)
	 write(20,*) psd(i)
	 write(20,*) epd(i)
	 do 313 j=1,nd(i)
	 write(20,*) tt(i,j),rc(i,j)
  313  continue
  312  continue

       read(11,*) ncol
	 write(20,*) ncol
	 read(11,1003) (wr(i),i=1,ncol)
 1003	 format(4f20.12)
       write(20,1004) (wr(i),i=1,ncol)
 1004  format(10f8.3)
	 do 400 i=1,ncol
	 read(11,1003) (br(i,j),j=1,ncol)
	 write(20,1005) (br(i,j),j=1,ncol)
 1005	 format(10f8.1)
  400	 continue
	 write(20,*)

	 close(10)
       close(11)

c       write(*,*) rfkd,rn
       if(nss.eq.0) go to 10
       if(nss.eq.1) go to 11
	 if(nss.eq.2) go to 12
	 if(nss.eq.3) go to 13
	 if(nss.eq.4) go to 14
	 if(nss.eq.7) go to 17
	 if(nss.eq.8) go to 18
	 if(nss.eq.9) go to 19
	 if(nss.eq.10) go to 20
	 if(nss.eq.11) go to 21
	 if(nss.eq.12) go to 22
10     continue
	 if (ns.eq.3) then 
	 xguess(1)=log10(rks)
       xguess(2)=log10(rdss)
	 xguess(3)=pfk
	 else
	 write(*,*) 'ns is not 3, though three dimentional search'
       stop
	 end if
	 go to 89	
11     continue
	 if (ns.eq.3) then 
	 xguess(1)=log10(rks)
       xguess(2)=log10(tort)
	 xguess(3)=pfk
	 else
	 write(*,*) 'ns is not 3, though three dimentional search'
       stop
	 end if
	 go to 89	
12     continue
	 if (ns.eq.3) then 
	 xguess(1)=log10(rkf)
       xguess(2)=log10(tort)
	 xguess(3)=log10(rdss)
	 else
	 write(*,*) 'ns is not 3, though three dimentional search'
       stop
	 end if
	 go to 89
13     continue
	 if (ns.eq.2) then
	 xguess(1)=log10(rkf)
       xguess(2)=log10(tort)
       else
	 write(*,*) 'ns is not 2, though two dimentional search'
       stop
	 end if
	 go to 89
14     continue
	 if (ns.eq.2) then
	 xguess(1)=log10(rkf)
	 xguess(2)=log10(rdss)
       else
	 write(*,*) 'ns is not 2, though two dimentional search'
       stop
	 end if
	 go to 89
17     continue
	 if (ns.eq.2) then
	 xguess(1)=log10(rks)
       xguess(2)=log10(tort)
       else
	 write(*,*) 'ns is not 2, though two dimentional search'
       stop
	 end if
	 go to 89
18     continue
	 if (ns.eq.2) then
	 xguess(1)=log10(rks)
       xguess(2)=log10(rdss)
       else
	 write(*,*) 'ns is not 2, though two dimentional search'
       stop
	 end if
	 go to 89
19     continue
	 if (ns.eq.2) then
       xguess(1)=log10(rdss)
       else
	 write(*,*) 'ns is not 2, though one dimentional search'
       stop
	 end if
	 go to 89
20    continue
	 if (ns.eq.2) then
       xguess(1)=log10(tort)
       else
	 write(*,*) 'ns is not 2, though one dimentional search'
       stop
	 end if
	 go to 89
21    continue
	 if (ns.eq.2) then
       xguess(1)=log10(rkf)
       else
	 write(*,*) 'ns is not 2, though one dimentional search'
       stop
	 end if
	 go to 89

22     continue

89     continue

       do 150 i=1,ns
       xscale(i)=1.0e-2
  150  continue
   
       fscale=1.0

       call du4inf(iparam,rparam)

	 iparam(3)=1000
	 if(nss.eq.12) iparam(3)=1

	 iparam(4)=4000

       call duminf(fcn,ns,xguess,xscale,fscale,iparam,rparam,xx,f)
       if(nss.eq.0) go to 30       
       if(nss.eq.1) go to 31
	 if(nss.eq.2) go to 32
	 if(nss.eq.3) go to 33
	 if(nss.eq.4) go to 34
	 if(nss.eq.7) go to 37
	 if(nss.eq.8) go to 38
	 if(nss.eq.9) go to 39
	 if(nss.eq.10) go to 40
	 if(nss.eq.11) go to 41
       
	 if(nss.eq.12) go to 42

30     continue 
       rks =10**xx(1)
       rdss=10**xx(2)
	 pfk =    xx(3)
	 go to 49	
31     continue 
       rks =10**xx(1)
       tort=10**xx(2)
	 pfk =    xx(3)
	 go to 49	
32     continue
       rkf =10**xx(1)
       tort=10**xx(2)
	 rdss=10**xx(3)
	 go to 49
33     continue
       rkf =10**xx(1)
       tort=10**xx(2)
	 go to 49
34     continue
       rkf =10**xx(1)	
	 rdss=10**xx(2)
	 go to 49
37     continue
       rks =10**xx(1)
       tort=10**xx(2)
	 go to 49
38     continue
       rks =10**xx(1)
       rdss=10**xx(2)
	 go to 49
39     continue
       rdss=10**xx(1)
	 go to 49
40     continue
       tort =10**xx(1)
	 go to 49
41     continue
       rkf =10**xx(1)
	 go to 49

42     continue

49     continue

       call umach(2,nout)

       write(*,1003)  rks,pfk,tort,rdss,rkf
	 write(*,*) 
	 write(20,1006) rks,pfk,tort,rdss,rkf
	 write(20,*)      
 1006  format(1h ,6(1pg12.4)) 

	 iflag=1     
	 call fcn(ns,xx,fmin)

       write(*,*) fmin
       write(20,*) fmin
       
	 close(20)              
       stop
       end
                    
       subroutine fcn (ns, xx, fmin)
c
	 IMPLICIT DOUBLE PRECISION (A-H,O-Z)

       parameter (mxparm=50)
	 parameter (ndim4=15)
	 parameter (ndim6=13)
	 parameter (ndim5=ndim4*ndim6*2)
	 parameter (ntest=15)

       dimension xx(ns)

       dimension param(mxparm),y(ndim5)
c	 -------for divpag----------
	 dimension aaa(1,1)
c      -------------------------

       dimension cinf(ntest),cc(ntest),nd(ntest),tt(ntest,10)
     &          ,rc(ntest,10),rfkd(ntest),rnd(ntest)
     &          ,psd(ntest),epd(ntest),d50t(ntest),d50a(ntest)       
       dimension acdose(ndim6),acdia(ntest,ndim6),radi(ndim6)
       dimension wr(ndim4),br(ndim4,ndim4)
       dimension acdosed(ntest,ndim6)

	 common/b1/nac
	 common/b2/acdose,acdia,radi
       common/b3/rfk,rn,pfk,ps,ep,co,rkf,rds,rdp,tort,
     &           rdss,rks,ceff,rceff
	 common/b4/ncol
       common/b5/wr,br
	 common/b33/rdpd,ps1,ep1,beta

	 common/b6/cinf,cc,tt,rc,psd,epd,d50t,d50a
	 common/b7/ndata,nd
	 common/b8/acdosed,tol,rfkd,rnd
	 common/b9/param11
	 common/b88/miter
       common/b10/iflag,nss
	 	 	 	                    
       external fcn2,fcnj,umach,sset

       if(nss.eq.0) go to 30
       if(nss.eq.1) go to 31
	 if(nss.eq.2) go to 32
	 if(nss.eq.3) go to 33
	 if(nss.eq.4) go to 34
	 if(nss.eq.7) go to 37
	 if(nss.eq.8) go to 38
	 if(nss.eq.9) go to 39
	 if(nss.eq.10) go to 40
	 if(nss.eq.11) go to 41
       if(nss.eq.12) go to 42

30     continue 
       rks =10**xx(1)
       rdss=10**xx(2)
	 pfk=    xx(3)
	 if(pfk.lt.0.0) pfk=0.0
	 if(pfk.gt.1.0) pfk=1.0
	 go to 49	
31     continue 
       rks =10**xx(1)
       tort=10**xx(2)
	 pfk1=    xx(3)
	 if(pfk.lt.0.0) pfk=0.0
	 if(pfk.gt.1.0) pfk=1.0
	 go to 49	
32     continue
       rkf =10**xx(1)
       tort=10**xx(2)
	 rdss=10**xx(3)
	 go to 49
33     continue
       rkf =10**xx(1)
       tort=10**xx(2)
	 go to 49
34     continue
       rkf =10**xx(1)	
	 rdss=10**xx(2)
	 go to 49
37     continue
       rks =10**xx(1)
       tort=10**xx(2)
	 go to 49
38     continue
       rks =10**xx(1)
       rdss=10**xx(2)
	 go to 49
39     continue
       rdss=10**xx(1)
	 go to 49
40     continue
       tort =10**xx(1)
	 go to 49
41     continue
       rkf = 10**xx(1)
	 go to 49

42     continue

49     continue

	 nequ=ndim5

c       write(*,*) 'solving RKG'
        write(*,*) 'Carbon dose  Co        time   C/Co-cal   C/Co-obs'

       fmin=0.0
	 do 302 i=1,ndata

       ps=psd(i)
	 ep=epd(i)

	 rdp=rdpd*ep/tort
       rds=rdss

c	write(*,*) i,ndata
c  ------for divpag----------------

          call sset(mxparm,0.0,param,1)

          ido=1
		param(1)=1e-10
		param(4)=10000000
	    param(10)=2
	    param(11)=param11

c		Select Gear's BDF method
          param(12)=2

c         MITER=1   for user-provided Jacobian
c         MITER=3   for Jacobian replaced matrics based on by directional derivative
          param(13)=miter

c         'A' matrix is not used, system is explicit
          param(19)=0

c ---------------------------------
	 co=cinf(i)
       rfk=rfkd(i)*pfk
	 rn=rnd(i)

       do 311 ij=1,nac
       acdose(ij)=acdosed(i,ij)*cc(i)
 	 radi(ij)=acdia(i,ij)/2.0

  311  continue

        do 505 ij=1,nequ
        y(ij)=0.0
  505   continue
		    	 
	  ntotal=0		 
        t=0.0
            rceff=1.0
c            write(*,*) i,t,rceff
	   do 303 ij=1,nd(i)

            tend=tt(i,ij)

          call divpag (ido,nequ,fcn2,fcnj,aaa,t,tend,tol,param,y)

 	      ceff=co
	      do 503 k=1,nac
	      wrq=0.0
		  do 501 j=1,ncol
	      jj=j+ncol*(k-1)
	      wrq=wrq+wr(j)*y(jj)
	      wrq=wrq+wr(j)*y(jj+ncol*nac)
  501       continue
            ceff=ceff-3.0*acdose(k)*wrq
  503       continue         

		  rceff=ceff/co

            write(*,1000) cc(i),co,t,rceff,rc(i,ij)
            if(iflag.eq.1) then
            write(20,*) 
     *       'Carbon dose  Co        time   C/Co-cal   C/Co-obs'
            write(20,1000) cc(i),co,t,rceff,rc(i,ij)
 1000       format(1h ,2(1pG10.3), 2x, 0pf8.0, 2x,2f8.4)
             else 
		  end if 

	   fmin=fmin+(rceff-rc(i,ij))**2

	  if(ij.eq.nd(i)) then 
	  ido=3
        call divpag (ido,nequ,fcn2,fcnj,aaa,t,tend,tol,param,y)
	  else
	  end if	
		       
  303  continue
       ntotal=ntotal+nd(i)
  302  continue                   
       fmin=(fmin/ntotal)**0.5

	 write(20,*) 
     * '  rks    pfk      tort      rdp     rdss    rkf       fmin'
	 write (*,*) 
     * '  rks    pfk      tort      rdp     rdss    rkf       fmin'
	 write(*,1001) rks,tort,pfk,rdp,rdss,rkf,fmin
	 write(20,1001) rks,tort,pfk,rdp,rdss,rkf,fmin
	 write(20,*) 
	 write (*,*) 
 1001  format(7(1g11.4))

            return
            end


       subroutine fcn2 (nn, t, y, yprime)
       IMPLICIT DOUBLE PRECISION (A-H,O-Z)

	 parameter (ndim4=15)
 	 parameter (ndim6=13)
	 parameter (ntest=15)
       dimension y(nn),yprime(nn)
       dimension acdose(ndim6),acdia(ntest,ndim6),radi(ndim6)
       dimension wr(ndim4),br(ndim4,ndim4)
       dimension q(ndim4,ndim6),p(ndim4,ndim6),cx(ndim4,ndim6)
       dimension dq(ndim4,ndim6),dp(ndim4,ndim6) 
	 	 
	 common/b1/nac
	 common/b2/acdose,acdia,radi
       common/b3/rfk,rn,pfk,ps,ep,co,rkf,rds,rdp,tort,
     &           rdss,rks,ceff,rceff
	 common/b4/ncol
       common/b5/wr,br
	 common/b33/rdpd,ps1,ep1,beta

        do 110 i=1,nn
	 yprime(i)=0.0
 110   continue

 	    ceff=co
 	    do 503 k=1,nac
		  wrq=0.0
		  do 501 j=1,ncol
	      jj=j+ncol*(k-1)
	      q(j,k)=y(jj)
		  wrq=wrq+wr(j)*q(j,k)
		  if (pfk.lt.0.9999999) then
	      p(j,k)=y(jj+nac*ncol)		   
		  wrq=wrq+wr(j)*p(j,k)
	      else
	      end if
  501       continue

            ceff=ceff-3.0*acdose(k)*wrq 
  503       continue  
    	 
	 do 99  k=1,nac
	 do 101 j=1,ncol
 	  cx(j,k)=dsign( (abs(q(j,k))/rfk)**rn, q(j,k) )
  101  continue    
   99  continue

	 ncol1=ncol-1   
	 do 107 m=1,nac
	 do 104 j=1,ncol1
	 dq(j,m)=0.0

	 do 105	k=1,ncol
	 dq(j,m)=dq(j,m)+br(j,k)*q(k,m)*rds/radi(m)**2
     &   	            +br(j,k)*cx(k,m)*rdp/radi(m)**2/ps
  105  continue
    
       if (pfk.lt.0.9999999) then
       dq(j,m)=dq(j,m)-rks*(q(j,m)/pfk-p(j,m)/(1.0-pfk))
       dp(j,m)=        rks*(q(j,m)/pfk-p(j,m)/(1.0-pfk))
	 else
	 end if

  104  continue

	 dq(ncol,m)=rkf/ps/wr(ncol)/3.0*(3.0/radi(m))*(ceff-cx(ncol,m))
	
	  do 106 j=1,ncol1
	  dq(ncol,m)=dq(ncol,m)-wr(j)/wr(ncol)*dq(j,m)
        if (pfk.lt.0.9999999) 
     &  dq(ncol,m)=dq(ncol,m)-wr(j)/wr(ncol)*dp(j,m)
  106   continue

        if (pfk.lt.0.9999999) then
        dq(ncol,m)=dq(ncol,m)-rks*(q(ncol,m)/pfk-p(ncol,m)/(1.0-pfk))
        dp(ncol,m)=           rks*(q(ncol,m)/pfk-p(ncol,m)/(1.0-pfk))
	  else
	  end if

  107   continue

  	 do 603 k=1,nac
	      do 601 j=1,ncol
	      jj=j+ncol*(k-1)
	      yprime(jj)=dq(j,k)
            if (pfk.lt.0.9999999) yprime(jj+nac*ncol)=dp(j,k)
  601       continue
  603  continue

       t=t

       return
       end

c    -----for divpag----------
       subroutine fcnj(nn,t,y,dy)
	 IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	 parameter (ndim4=15)
 	 parameter (ndim6=13)
	 parameter (ntest=15)
	 	      
       dimension y(nn),dy(nn,nn)
       dimension acdose(ndim6),acdia(ntest,ndim6),radi(ndim6)
       dimension wr(ndim4),br(ndim4,ndim4)
       dimension q(ndim4,ndim6),p(ndim4,ndim6),cx(ndim4,ndim6)
       dimension qq(ndim4,ndim6,ndim4,ndim6)
     &          ,pp(ndim4,ndim6,ndim4,ndim6)
     &          ,qp(ndim4,ndim6,ndim4,ndim6)
     &          ,pq(ndim4,ndim6,ndim4,ndim6)
     	  	 
	 common/b1/nac
	 common/b2/acdose,acdia,radi
       common/b3/rfk,rn,pfk,ps,ep,co,rkf,rds,rdp,tort,
     &           rdss,rks,ceff,rceff
	 common/b4/ncol
       common/b5/wr,br
	 common/b33/rdpd,ps1,ep1,beta

       external sset

	 call sset (nn**2, 0.0, dy, 1)
   
       do 110 i=1,nn
	 do 111 j=1,nn
	 dy(i,j)=0.0
 111   continue
 110   continue

 	    do 120 k=1,nac
		  do 121 j=1,ncol
	      jj=j+ncol*(k-1)
	      q(j,k)=y(jj)
	      if (pfk.lt.0.9999999) p(j,k)=y(jj+nac*ncol)
  121       continue
  120       continue  
 
	 ncol1=ncol-1

	 do 201 m=1,nac

	 do 301 i=1,ncol1
	 do 302	n=1,nac
       do 303 j=1,ncol

       if(m.eq.n) then
c-----------------------------------------------------
	 qrn=dsign(abs(q(j,m))**(rn-1.0),q(j,m))
       qq(i,m,j,n)=rds/radi(m)**2*br(i,j)+
     &  +rdp/radi(m)**2/ps*rn*br(i,j)*qrn/rfk**rn
c-----------------------------------------------------
       else
       qq(i,m,j,n)=0.0
       end if

  303  continue
  302  continue
  301  continue

	 do 311	n=1,nac
       do 312 j=1,ncol1

       if(m.eq.n) then
c-----------------------------------------------------
	 qrn=dsign(abs(q(j,m))**(rn-1.0),q(j,m))
 	 qq(ncol,m,j,n)
     &   =rkf*(3.0/radi(m))/ps/wr(ncol)*(-wr(j)*acdose(m))
       do 313 k=1,ncol1
       qq(ncol,m,j,n)=qq(ncol,m,j,n)
     &               -(rds+rdp/ps*rn*qrn/rfk**rn)
     &                /wr(ncol)/radi(m)**2*wr(k)*br(k,j)
  313  continue
c-----------------------------------------------------
       else
	 qq(ncol,m,j,n)
     &   =rkf*(3.0/radi(m))/ps/wr(ncol)*(-wr(j)*acdose(n))
	 end if

  312  continue
  311  continue

	 do 321	n=1,nac
       if(m.eq.n) then
c-----------------------------------------------------
	 qrn=dsign(abs(q(ncol,m))**(rn-1.0),q(ncol,m))
	 qq(ncol,m,ncol,n)=
     &   rkf*(3.0/radi(m))/3.0/ps/wr(ncol)
     &  *(-3.0*acdose(m)*wr(ncol)-rn/rfk**rn*qrn)

       do 322 k=1,ncol1
       qq(ncol,m,ncol,n)=qq(ncol,m,ncol,n)
     &               -(rds+rdp/ps*rn*qrn/rfk**rn)
     &                /wr(ncol)/radi(m)**2*wr(k)*br(k,ncol)
  322  continue
c-----------------------------------------------------
       else
	 qq(ncol,m,ncol,n)
     &   =rkf*(3.0/radi(m))/ps/wr(ncol)*(-wr(ncol)*acdose(n))
	 end if

  321   continue

  201   continue

       if (pfk.lt.0.9999999) then

	 do 401 m=1,nac

	 do 501 i=1,ncol
	 do 502	n=1,nac
       do 503 j=1,ncol
       qp(i,m,j,n)=0.0
       pq(i,m,j,n)=0.0
       pp(i,m,j,n)=0.0
  503  continue
  502  continue
       qq(i,m,i,m)=qq(i,m,i,m)-rks/pfk
	 qp(i,m,i,m)=            rks/(1.0-pfk)
	 pq(i,m,i,m)=            rks/pfk
	 pp(i,m,i,m)=           -rks/(1.0-pfk)
  501  continue

	 do 511	n=1,nac
       do 512 j=1,ncol
	 qp(ncol,m,j,n)=qp(ncol,m,j,n)
     &   +rkf*(3.0/radi(m))/ps/wr(ncol)*(-wr(j)*acdose(n))
  512  continue
  511  continue

  401   continue

       else
	 end if

   	 do 601 m=1,nac
	      do 602 i=1,ncol
       do 603 n=1,nac
	      do 604 j=1,ncol
	      ii=i+ncol*(m-1)
	      jj=j+ncol*(n-1)
	      dy(ii,jj)=qq(i,m,j,n)
  604  continue
  603  continue
  602  continue
  601  continue

	 if (pfk.lt.0.9999999) then
   	 do 701 m=1,nac
	      do 702 i=1,ncol
       do 703 n=1,nac
	      do 704 j=1,ncol
	      ii=i+ncol*(m-1)
	      jj=j+ncol*(n-1)
	      dy(ii,         jj+nac*ncol)=qp(i,m,j,n)
	      dy(ii+nac*ncol,jj         )=pq(i,m,j,n)
	      dy(ii+nac*ncol,jj+nac*ncol)=pp(i,m,j,n)
  704  continue
  703  continue
  702  continue
  701  continue
       else
	 end if

       t=t

	 return
	 end

	                        